@@ -1,29 +1,27 @@
 """ Test cogapp.
     http://nedbatchelder.com/code/cog
 
-    Copyright 2004-2015, Ned Batchelder.
+    Copyright 2004-2019, Ned Batchelder.
 """
 
 from __future__ import absolute_import
 
-import os, os.path, random, re, shutil, stat, sys, tempfile
+import os
+import os.path
+import random
+import re
+import shutil
+import stat
+import sys
+import tempfile
+import threading
 
-# Use unittest2 if it's available, otherwise unittest.  This gives us
-# back-ported features for 2.6.
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
-
-from .backward import StringIO, to_bytes, b
+from .backward import StringIO, to_bytes, TestCase, PY3
 from .cogapp import Cog, CogOptions, CogGenerator
-from .cogapp import CogError, CogUsageError, CogGeneratedError
-from .cogapp import usage, __version__
-from .whiteutils import reindentBlock
+from .cogapp import CogError, CogUsageError, CogGeneratedError, CogUserException
+from .cogapp import usage, __version__, main
 from .makefiles import *
-
-
-TestCase = unittest.TestCase
+from .whiteutils import reindentBlock
 
 
 class CogTestsInMemory(TestCase):
@@ -146,6 +144,53 @@ class CogTestsInMemory(TestCase):
         infile = reindentBlock(infile)
         self.assertEqual(Cog().processString(infile), infile)
 
+    def testTrimBlankLinesWithLastPartial(self):
+        infile = """\
+            //[[[cog
+            cog.out("This is line one\\n", trimblanklines=True)
+            cog.out("\\nLine two\\nLine three", trimblanklines=True)
+            //]]]
+            This is line one
+            Line two
+            Line three
+            //[[[end]]]
+            """
+
+        infile = reindentBlock(infile)
+        self.assertEqual(Cog().processString(infile), infile)
+
+    def testCogOutDedent(self):
+        infile = """\
+            //[[[cog
+            cog.out("This is the first line\\n")
+            cog.out('''
+                This is dedent=True 1
+                 This is dedent=True 2
+            ''', dedent=True, trimblanklines=True)
+            cog.out('''
+                This is dedent=False 1
+                 This is dedent=False 2
+            ''', dedent=False, trimblanklines=True)
+            cog.out('''
+                This is dedent=default 1
+                 This is dedent=default 2
+            ''', trimblanklines=True)
+            cog.out("This is the last line\\n")
+            //]]]
+            This is the first line
+            This is dedent=True 1
+             This is dedent=True 2
+                This is dedent=False 1
+                 This is dedent=False 2
+                This is dedent=default 1
+                 This is dedent=default 2
+            This is the last line
+            //[[[end]]]
+            """
+
+        infile = reindentBlock(infile)
+        self.assertEqual(Cog().processString(infile), infile)
+
     def test22EndOfLine(self):
         # In Python 2.2, this cog file was not parsing because the
         # last line is indented but didn't end with a newline.
@@ -295,6 +340,7 @@ class CogTestsInMemory(TestCase):
                 cog.outl("x")
                 cog.outl()
                 cog.outl("y")
+                cog.out()   # Also optional, a complete no-op.
                 cog.outl(trimblanklines=True)
                 cog.outl("z")
             ]]]
@@ -357,7 +403,7 @@ class CogTestsInMemory(TestCase):
             [[[end]]]
             last line
             """
-        with self.assertRaisesRegexp(CogError, r"infile.txt\(2\): Cog code markers inverted"):
+        with self.assertRaisesRegex(CogError, r"^infile.txt\(2\): Cog code markers inverted$"):
              Cog().processString(reindentBlock(infile), "infile.txt")
 
     def testSharingGlobals(self):
@@ -394,7 +440,7 @@ class CogTestsInMemory(TestCase):
             [[[end]]]
             """
         infile = reindentBlock(infile)
-        with self.assertRaisesRegexp(AssertionError, "Oops"):
+        with self.assertRaisesRegex(CogUserException, "AssertionError: Oops"):
             Cog().processString(infile)
 
     def testCogPrevious(self):
@@ -478,7 +524,7 @@ class FileStructureTests(TestCase):
 
     def isBad(self, infile, msg=None):
         infile = reindentBlock(infile)
-        with self.assertRaisesRegexp(CogError, re.escape(msg)):
+        with self.assertRaisesRegex(CogError, "^"+re.escape(msg)+"$"):
             Cog().processString(infile, 'infile.txt')
 
     def testBeginNoEnd(self):
@@ -616,7 +662,7 @@ class CogErrorTests(TestCase):
             """
 
         infile = reindentBlock(infile)
-        with self.assertRaisesRegexp(CogGeneratedError, "This ain't right!"):
+        with self.assertRaisesRegex(CogGeneratedError, "^This ain't right!$"):
             Cog().processString(infile)
 
     def testErrorNoMsg(self):
@@ -626,7 +672,7 @@ class CogErrorTests(TestCase):
             """
 
         infile = reindentBlock(infile)
-        with self.assertRaisesRegexp(CogGeneratedError, "Error raised by cog generator."):
+        with self.assertRaisesRegex(CogGeneratedError, "^Error raised by cog generator.$"):
             Cog().processString(infile)
 
     def testNoErrorIfErrorNotCalled(self):
@@ -761,9 +807,9 @@ class ArgumentHandlingTests(TestCaseWithTempDir):
         self.assertEqual(self.cog.main(['argv0', '-j']), 2)
         output = self.output.getvalue()
         self.assertIn("option -j not recognized", output)
-        with self.assertRaises(CogUsageError):
+        with self.assertRaisesRegex(CogUsageError, r"^No files to process$"):
             self.cog.callableMain(['argv0'])
-        with self.assertRaises(CogUsageError):
+        with self.assertRaisesRegex(CogUsageError, r"^option -j not recognized$"):
             self.cog.callableMain(['argv0', '-j'])
 
     def testNoDashOAndAtFile(self):
@@ -774,7 +820,7 @@ class ArgumentHandlingTests(TestCaseWithTempDir):
             }
 
         makeFiles(d)
-        with self.assertRaises(CogUsageError):
+        with self.assertRaisesRegex(CogUsageError, r"^Can't use -o with @file$"):
             self.cog.callableMain(['argv0', '-o', 'foo', '@cogfiles.txt'])
 
     def testDashV(self):
@@ -803,7 +849,7 @@ class ArgumentHandlingTests(TestCaseWithTempDir):
             }
 
         makeFiles(d)
-        with self.assertRaises(CogUsageError):
+        with self.assertRaisesRegex(CogUsageError, r"^Can't use -o with -r \(they are opposites\)$"):
             self.cog.callableMain(['argv0', '-o', 'foo', '-r', 'cogfile.txt'])
 
     def testDashZ(self):
@@ -831,25 +877,113 @@ class ArgumentHandlingTests(TestCaseWithTempDir):
             }
 
         makeFiles(d)
-        with self.assertRaisesRegexp(CogError, re.escape("test.cog(6): Missing '[[[end]]]' before end of file.")):
+        with self.assertRaisesRegex(CogError, r"^test.cog\(6\): Missing '\[\[\[end\]\]\]' before end of file.$"):
             self.cog.callableMain(['argv0', '-r', 'test.cog'])
         self.newCog()
         self.cog.callableMain(['argv0', '-r', '-z', 'test.cog'])
         self.assertFilesSame('test.cog', 'test.out')
 
     def testBadDashD(self):
-        with self.assertRaises(CogUsageError):
+        with self.assertRaisesRegex(CogUsageError, r"^-D takes a name=value argument$"):
             self.cog.callableMain(['argv0', '-Dfooey', 'cog.txt'])
-        with self.assertRaises(CogUsageError):
+        with self.assertRaisesRegex(CogUsageError, r"^-D takes a name=value argument$"):
             self.cog.callableMain(['argv0', '-D', 'fooey', 'cog.txt'])
 
     def testBadMarkers(self):
-        with self.assertRaises(CogUsageError):
+        with self.assertRaisesRegex(CogUsageError, r"^--markers requires 3 values separated by spaces, could not parse 'X'$"):
             self.cog.callableMain(['argv0', '--markers=X'])
-        with self.assertRaises(CogUsageError):
+        with self.assertRaisesRegex(CogUsageError, r"^--markers requires 3 values separated by spaces, could not parse 'A B C D'$"):
             self.cog.callableMain(['argv0', '--markers=A B C D'])
 
 
+class TestMain(TestCaseWithTempDir):
+    def setUp(self):
+        super(TestMain, self).setUp()
+        self.old_argv = sys.argv[:]
+        self.old_stderr = sys.stderr
+        sys.stderr = StringIO()
+
+    def tearDown(self):
+        sys.stderr = self.old_stderr
+        sys.argv = self.old_argv
+        sys.modules.pop('mycode', None)
+        super(TestMain, self).tearDown()
+
+    def test_main_function(self):
+        sys.argv = ["argv0", "-Z"]
+        ret = main()
+        self.assertEqual(ret, 2)
+        stderr = sys.stderr.getvalue()
+        self.assertEqual(stderr, 'option -Z not recognized\n(for help use -?)\n')
+
+    files = {
+        'test.cog': """\
+            //[[[cog
+            def func():
+                import mycode
+                mycode.boom()
+            //]]]
+            //[[[end]]]
+            -----
+            //[[[cog
+            func()
+            //]]]
+            //[[[end]]]
+            """,
+
+        'mycode.py': """\
+            def boom():
+                [][0]
+            """,
+        }
+
+    def test_error_report(self):
+        self.check_error_report()
+
+    def test_error_report_with_prologue(self):
+        self.check_error_report("-p", "#1\n#2")
+
+    def check_error_report(self, *args):
+        """Check that the error report is right."""
+        makeFiles(self.files)
+        sys.argv = ["argv0"] + list(args) + ["-r", "test.cog"]
+        main()
+        expected = reindentBlock("""\
+            Traceback (most recent call last):
+              File "test.cog", line 9, in <module>
+                func()
+              File "test.cog", line 4, in func
+                mycode.boom()
+              File "MYCODE", line 2, in boom
+                [][0]
+            IndexError: list index out of range
+            """)
+        if PY3:
+            expected = expected.replace("MYCODE", os.path.abspath("mycode.py"))
+        else:
+            expected = expected.replace("MYCODE", "mycode.py")
+        assert expected == sys.stderr.getvalue()
+
+    def test_error_in_prologue(self):
+        makeFiles(self.files)
+        sys.argv = ["argv0", "-p", "import mycode; mycode.boom()", "-r", "test.cog"]
+        main()
+        expected = reindentBlock("""\
+            Traceback (most recent call last):
+              File "<prologue>", line 1, in <module>
+                import mycode; mycode.boom()
+              File "MYCODE", line 2, in boom
+                [][0]
+            IndexError: list index out of range
+            """)
+        if PY3:
+            expected = expected.replace("MYCODE", os.path.abspath("mycode.py"))
+        else:
+            expected = expected.replace("MYCODE", "mycode.py")
+        assert expected == sys.stderr.getvalue()
+
+
+
 class TestFileHandling(TestCaseWithTempDir):
 
     def testSimple(self):
@@ -884,6 +1018,70 @@ class TestFileHandling(TestCaseWithTempDir):
         output = self.output.getvalue()
         self.assertIn("(changed)", output)
 
+    def testWildcards(self):
+        d = {
+            'test.cog': """\
+                // This is my C++ file.
+                //[[[cog
+                fnames = ['DoSomething', 'DoAnotherThing', 'DoLastThing']
+                for fn in fnames:
+                    cog.outl("void %s();" % fn)
+                //]]]
+                //[[[end]]]
+                """,
+
+            'test2.cog': """\
+                // This is my C++ file.
+                //[[[cog
+                fnames = ['DoSomething', 'DoAnotherThing', 'DoLastThing']
+                for fn in fnames:
+                    cog.outl("void %s();" % fn)
+                //]]]
+                //[[[end]]]
+                """,
+
+            'test.out': """\
+                // This is my C++ file.
+                //[[[cog
+                fnames = ['DoSomething', 'DoAnotherThing', 'DoLastThing']
+                for fn in fnames:
+                    cog.outl("void %s();" % fn)
+                //]]]
+                void DoSomething();
+                void DoAnotherThing();
+                void DoLastThing();
+                //[[[end]]]
+                """,
+
+            'not_this_one.cog': """\
+                // This is my C++ file.
+                //[[[cog
+                fnames = ['DoSomething', 'DoAnotherThing', 'DoLastThing']
+                for fn in fnames:
+                    cog.outl("void %s();" % fn)
+                //]]]
+                //[[[end]]]
+                """,
+
+            'not_this_one.out': """\
+                // This is my C++ file.
+                //[[[cog
+                fnames = ['DoSomething', 'DoAnotherThing', 'DoLastThing']
+                for fn in fnames:
+                    cog.outl("void %s();" % fn)
+                //]]]
+                //[[[end]]]
+                """,
+            }
+
+        makeFiles(d)
+        self.cog.callableMain(['argv0', '-r', 't*.cog'])
+        self.assertFilesSame('test.cog', 'test.out')
+        self.assertFilesSame('test2.cog', 'test.out')
+        self.assertFilesSame('not_this_one.cog', 'not_this_one.out')
+        output = self.output.getvalue()
+        self.assertIn("(changed)", output)
+
     def testOutputFile(self):
         # -o sets the output file.
         d = {
@@ -912,8 +1110,8 @@ class TestFileHandling(TestCaseWithTempDir):
             }
 
         makeFiles(d)
-        self.cog.callableMain(['argv0', '-o', 'test.cogged', 'test.cog'])
-        self.assertFilesSame('test.cogged', 'test.out')
+        self.cog.callableMain(['argv0', '-o', 'in/a/dir/test.cogged', 'test.cog'])
+        self.assertFilesSame('in/a/dir/test.cogged', 'test.out')
 
     def testAtFile(self):
         d = {
@@ -1045,15 +1243,15 @@ class TestFileHandling(TestCaseWithTempDir):
 
             'cogfiles.txt': """\
                 # Please run cog
-                both.cog -o both.one -s ' // ONE' -D one=x
-                both.cog -o both.two -s ' // TWO' -D two=x
+                both.cog -o in/a/dir/both.one -s ' // ONE' -D one=x
+                both.cog -o in/a/dir/both.two -s ' // TWO' -D two=x
                 """
             }
 
         makeFiles(d)
         self.cog.callableMain(['argv0', '@cogfiles.txt'])
-        self.assertFilesSame('both.one', 'one.out')
-        self.assertFilesSame('both.two', 'two.out')
+        self.assertFilesSame('in/a/dir/both.one', 'one.out')
+        self.assertFilesSame('in/a/dir/both.two', 'two.out')
 
     def testAtFileWithBadArgCombo(self):
         d = {
@@ -1073,7 +1271,7 @@ class TestFileHandling(TestCaseWithTempDir):
             }
 
         makeFiles(d)
-        with self.assertRaises(CogUsageError):
+        with self.assertRaisesRegex(CogUsageError, r"^Can't use -d with -r \(or you would delete all your source!\)$"):
             self.cog.callableMain(['argv0', '-r', '@cogfiles.txt'])
 
     def testAtFileWithTrickyFilenames(self):
@@ -1214,25 +1412,25 @@ class CogTestCharacterEncoding(TestCaseWithTempDir):
 
     def testSimple(self):
         d = {
-            'test.cog': b("""\
+            'test.cog': b"""\
                 // This is my C++ file.
                 //[[[cog
                 cog.outl("// Unicode: \xe1\x88\xb4 (U+1234)")
                 //]]]
                 //[[[end]]]
-                """),
+                """,
 
-            'test.out': b("""\
+            'test.out': b"""\
                 // This is my C++ file.
                 //[[[cog
                 cog.outl("// Unicode: \xe1\x88\xb4 (U+1234)")
                 //]]]
                 // Unicode: \xe1\x88\xb4 (U+1234)
                 //[[[end]]]
-                """),
+                """.replace(b"\n", os.linesep.encode()),
             }
 
-        makeFiles(d)
+        makeFiles(d, bytes=True)
         self.cog.callableMain(['argv0', '-r', 'test.cog'])
         self.assertFilesSame('test.cog', 'test.out')
         output = self.output.getvalue()
@@ -1240,25 +1438,25 @@ class CogTestCharacterEncoding(TestCaseWithTempDir):
 
     def testFileEncodingOption(self):
         d = {
-            'test.cog': b("""\
+            'test.cog': b"""\
                 // \xca\xee\xe4\xe8\xf0\xe2\xea\xe0 Windows
                 //[[[cog
                 cog.outl("\xd1\xfa\xe5\xf8\xfc \xe5\xf9\xb8 \xfd\xf2\xe8\xf5 \xec\xff\xe3\xea\xe8\xf5 \xf4\xf0\xe0\xed\xf6\xf3\xe7\xf1\xea\xe8\xf5 \xe1\xf3\xeb\xee\xea \xe4\xe0 \xe2\xfb\xef\xe5\xe9 \xf7\xe0\xfe")
                 //]]]
                 //[[[end]]]
-                """),
+                """,
 
-            'test.out': b("""\
+            'test.out': b"""\
                 // \xca\xee\xe4\xe8\xf0\xe2\xea\xe0 Windows
                 //[[[cog
                 cog.outl("\xd1\xfa\xe5\xf8\xfc \xe5\xf9\xb8 \xfd\xf2\xe8\xf5 \xec\xff\xe3\xea\xe8\xf5 \xf4\xf0\xe0\xed\xf6\xf3\xe7\xf1\xea\xe8\xf5 \xe1\xf3\xeb\xee\xea \xe4\xe0 \xe2\xfb\xef\xe5\xe9 \xf7\xe0\xfe")
                 //]]]
                 \xd1\xfa\xe5\xf8\xfc \xe5\xf9\xb8 \xfd\xf2\xe8\xf5 \xec\xff\xe3\xea\xe8\xf5 \xf4\xf0\xe0\xed\xf6\xf3\xe7\xf1\xea\xe8\xf5 \xe1\xf3\xeb\xee\xea \xe4\xe0 \xe2\xfb\xef\xe5\xe9 \xf7\xe0\xfe
                 //[[[end]]]
-                """),
+                """.replace(b"\n", os.linesep.encode()),
             }
 
-        makeFiles(d)
+        makeFiles(d, bytes=True)
         self.cog.callableMain(['argv0', '-n', 'cp1251', '-r', 'test.cog'])
         self.assertFilesSame('test.cog', 'test.out')
         output = self.output.getvalue()
@@ -1272,7 +1470,7 @@ class TestCaseWithImports(TestCaseWithTempDir):
     """
 
     def setUp(self):
-        TestCaseWithTempDir.setUp(self)
+        super(TestCaseWithImports, self).setUp()
         self.sysmodulekeys = list(sys.modules)
 
     def tearDown(self):
@@ -1283,7 +1481,7 @@ class TestCaseWithImports(TestCaseWithTempDir):
             ]
         for modname in modstoscrub:
             del sys.modules[modname]
-        TestCaseWithTempDir.tearDown(self)
+        super(TestCaseWithImports, self).tearDown()
 
 
 class CogIncludeTests(TestCaseWithImports):
@@ -1336,7 +1534,8 @@ class CogIncludeTests(TestCaseWithImports):
     def testNeedIncludePath(self):
         # Try it without the -I, to see that an ImportError happens.
         makeFiles(self.dincludes)
-        with self.assertRaises(ImportError):
+        msg = "(ImportError|ModuleNotFoundError): No module named '?mymodule'?"
+        with self.assertRaisesRegex(CogUserException, msg):
             self.cog.callableMain(['argv0', '-r', 'test.cog'])
 
     def testIncludePath(self):
@@ -1394,17 +1593,17 @@ class CogIncludeTests(TestCaseWithImports):
         self.assertEqual(oldsyspath, sys.path)
         # Is it unchanged for a failed run?
         self.newCog()
-        with self.assertRaises(CogError):
+        with self.assertRaisesRegex(CogError, r"^Oh no!$"):
             self.cog.callableMain(['argv0', '-r', 'bad.cog'])
         self.assertEqual(oldsyspath, sys.path)
         # Is it unchanged for a failed run with includes?
         self.newCog()
-        with self.assertRaises(CogError):
+        with self.assertRaisesRegex(CogError, r"^Oh no!$"):
             self.cog.callableMain(['argv0', '-r', '-I', 'xyzzy', 'bad.cog'])
         self.assertEqual(oldsyspath, sys.path)
         # Is it unchanged for a failed run with two includes?
         self.newCog()
-        with self.assertRaises(CogError):
+        with self.assertRaisesRegex(CogError, r"^Oh no!$"):
             self.cog.callableMain(['argv0', '-r', '-I', 'xyzzy', '-I', 'quux', 'bad.cog'])
         self.assertEqual(oldsyspath, sys.path)
 
@@ -1711,7 +1910,6 @@ class CogTestsInFiles(TestCaseWithTempDir):
         self.assertEqual(output, "--[[[cog cog.outl('Wow') ]]]\nWow\n--[[[end]]]\n")
         self.assertEqual(outerr, "")
 
-
     def testSuffixOutputLines(self):
         d = {
             'test.cog': """\
@@ -1777,6 +1975,64 @@ class CogTestsInFiles(TestCaseWithTempDir):
         self.cog.callableMain(['argv0', '-z', '-r', '-s', r' /\n*+([)]><', 'test.cog'])
         self.assertFilesSame('test.cog', 'test.out')
 
+    def testPrologue(self):
+        d = {
+            'test.cog': """\
+                Some text.
+                //[[[cog cog.outl(str(math.sqrt(2))[:12])]]]
+                //[[[end]]]
+                epilogue.
+                """,
+
+            'test.out': """\
+                Some text.
+                //[[[cog cog.outl(str(math.sqrt(2))[:12])]]]
+                1.4142135623
+                //[[[end]]]
+                epilogue.
+                """,
+            }
+
+        makeFiles(d)
+        self.cog.callableMain(['argv0', '-r', '-p', 'import math', 'test.cog'])
+        self.assertFilesSame('test.cog', 'test.out')
+
+    def testThreads(self):
+        # Test that the implicitly imported cog module is actually different for
+        # different threads.
+        numthreads = 20
+
+        d = {}
+        for i in range(numthreads):
+            d['f{}.cog'.format(i)] = (
+                "x\n" * i +
+                "[[[cog\n" +
+                "assert cog.firstLineNum == int(FIRST) == {}\n".format(i+1) +
+                "]]]\n" +
+                "[[[end]]]\n"
+                )
+        makeFiles(d)
+
+        results = []
+
+        def thread_main(num):
+            try:
+                ret = Cog().main(
+                    ['cog.py', '-r', '-D', 'FIRST={}'.format(num+1), 'f{}.cog'.format(num)]
+                    )
+                assert ret == 0
+            except Exception as exc:    # pragma: no cover (only happens on test failure)
+                results.append(exc)
+            else:
+                results.append(None)
+
+        ts = [threading.Thread(target=thread_main, args=(i,)) for i in range(numthreads)]
+        for t in ts:
+            t.start()
+        for t in ts:
+            t.join()
+        assert results == [None] * numthreads
+
 
 class WritabilityTests(TestCaseWithTempDir):
 
@@ -1811,7 +2067,7 @@ class WritabilityTests(TestCaseWithTempDir):
         cmd_w_asterisk = 'chmod +w *'
 
     def setUp(self):
-        TestCaseWithTempDir.setUp(self)
+        super(WritabilityTests, self).setUp()
         makeFiles(self.d)
         self.testcog = os.path.join(self.tempdir, 'test.cog')
         os.chmod(self.testcog, stat.S_IREAD)   # Make the file readonly.
@@ -1819,10 +2075,10 @@ class WritabilityTests(TestCaseWithTempDir):
 
     def tearDown(self):
         os.chmod(self.testcog, stat.S_IWRITE)   # Make the file writable again.
-        TestCaseWithTempDir.tearDown(self)
+        super(WritabilityTests, self).tearDown()
 
     def testReadonlyNoCommand(self):
-        with self.assertRaisesRegexp(CogError, "Can't overwrite test.cog"):
+        with self.assertRaisesRegex(CogError, "^Can't overwrite test.cog$"):
             self.cog.callableMain(['argv0', '-r', 'test.cog'])
         assert not os.access(self.testcog, os.W_OK)
 
@@ -1837,7 +2093,7 @@ class WritabilityTests(TestCaseWithTempDir):
         assert os.access(self.testcog, os.W_OK)
 
     def testReadonlyWithIneffectualCommand(self):
-        with self.assertRaisesRegexp(CogError, "Couldn't make test.cog writable"):
+        with self.assertRaisesRegex(CogError, "^Couldn't make test.cog writable$"):
             self.cog.callableMain(['argv0', '-r', '-w', 'echo %s', 'test.cog'])
         assert not os.access(self.testcog, os.W_OK)
 
@@ -2002,23 +2258,23 @@ class ChecksumTests(TestCaseWithTempDir):
             }
 
         makeFiles(d)
-        with self.assertRaisesRegexp(CogError,
-            r"cog1.txt\(9\): Output has been edited! Delete old checksum to unprotect."):
+        with self.assertRaisesRegex(CogError,
+            r"^cog1.txt\(9\): Output has been edited! Delete old checksum to unprotect.$"):
             self.cog.callableMain(['argv0', '-c', "cog1.txt"])
-        with self.assertRaisesRegexp(CogError,
-            r"cog2.txt\(9\): Output has been edited! Delete old checksum to unprotect."):
+        with self.assertRaisesRegex(CogError,
+            r"^cog2.txt\(9\): Output has been edited! Delete old checksum to unprotect.$"):
             self.cog.callableMain(['argv0', '-c', "cog2.txt"])
-        with self.assertRaisesRegexp(CogError,
-            r"cog3.txt\(10\): Output has been edited! Delete old checksum to unprotect."):
+        with self.assertRaisesRegex(CogError,
+            r"^cog3.txt\(10\): Output has been edited! Delete old checksum to unprotect.$"):
             self.cog.callableMain(['argv0', '-c', "cog3.txt"])
-        with self.assertRaisesRegexp(CogError,
-            r"cog4.txt\(9\): Output has been edited! Delete old checksum to unprotect."):
+        with self.assertRaisesRegex(CogError,
+            r"^cog4.txt\(9\): Output has been edited! Delete old checksum to unprotect.$"):
             self.cog.callableMain(['argv0', '-c', "cog4.txt"])
-        with self.assertRaisesRegexp(CogError,
-            r"cog5.txt\(10\): Output has been edited! Delete old checksum to unprotect."):
+        with self.assertRaisesRegex(CogError,
+            r"^cog5.txt\(10\): Output has been edited! Delete old checksum to unprotect.$"):
             self.cog.callableMain(['argv0', '-c', "cog5.txt"])
-        with self.assertRaisesRegexp(CogError,
-            r"cog6.txt\(6\): Output has been edited! Delete old checksum to unprotect."):
+        with self.assertRaisesRegex(CogError,
+            r"^cog6.txt\(6\): Output has been edited! Delete old checksum to unprotect.$"):
             self.cog.callableMain(['argv0', '-c', "cog6.txt"])
 
     def testArgvIsntModified(self):
@@ -2149,7 +2405,7 @@ class BlakeTests(TestCaseWithTempDir):
             }
 
         makeFiles(d)
-        with self.assertRaises(CogUsageError):
+        with self.assertRaisesRegex(CogUsageError, r"^Can't use -d with -r \(or you would delete all your source!\)$"):
             self.cog.callableMain(['argv0', '-r', '-d', 'test.cog'])
 
     def testSettingGlobals(self):
@@ -2214,7 +2470,7 @@ class ErrorCallTests(TestCaseWithTempDir):
         self.cog.main(['argv0', '-r', 'error.cog'])
         output = self.output.getvalue()
         msg = 'Actual output:\n' + output
-        self.assert_(output.startswith("Cogging error.cog\nTraceback (most recent"), msg)
+        self.assertTrue(output.startswith("Cogging error.cog\nTraceback (most recent"), msg)
         self.assertIn("RuntimeError: Hey!", output)
 
 
